#!/bin/bash

me=$(readlink -f $0)
BASE=$(dirname ${me})

if test "$(lsb_release -is)" != "CentOS";then
    read -p "Wrong OS! Please download the Ubuntu or combined update from http://www.ferrari-electronic.de. [Abort]" answer
    exit 2
fi

featureversion=$(featureversion)

if [ ! -z "${featureversion}" ]; then
    if [ -e /tmp/gateconfig.featureversion ]; then
        gateconfigversion=$(cat /tmp/gateconfig.featureversion)
        if [ -z "${gateconfigversion}" -o ${gateconfigversion} -lt ${featureversion} ]; then
            echo "***********************************************************************************"
            echo "* OfficeMaster GateConfiguration too old. Please download new version"
            echo "* http://www.ferrari-electronic.de/fileadmin/download_center/OfficeMasterHardware/Setup-OfficeMaster-Gate-Config.exe"
            echo "***********************************************************************************"
            read -p "OfficeMaster GateConfiguration too old. Please download new version. [Abort/Default=Abort]!" answer
            sleep 5
            echo "## Installation failed ## ..."
            echo "FLASH_FAILED" # Old gateConfig
            touch /tmp/flashfailed
            exit 1
        fi
    else
        if [ -e /var/config.active -o "${HISTFILE}" != "" ]; then
            echo "***********************************************************************************"
            echo "* OfficeMaster GateConfiguration too old. Please download new version"
            echo "* http://www.ferrari-electronic.de/fileadmin/download_center/OfficeMasterHardware/Setup-OfficeMaster-Gate-Config.exe"
            echo "***********************************************************************************"
            read -p "OfficeMaster GateConfiguration too old. Please download new version. [Abort/Default=Abort]!" answer
            sleep 5
            echo "## Installation failed ## ..."
            echo "FLASH_FAILED" # Old gateConfig
            touch /tmp/flashfailed
            exit 2
        fi
    fi
fi


modprobe i2c-dev 2>/dev/null
modprobe tdm_dma 2>/dev/null
modprobe tdm_i2c 2>/dev/null

echo "Check hardware for FPGA update ..."

if ! which i2crdtype > /dev/null; then
    echo "Error: i2crdtype misssing. Perform omgupdate first."
    exit 2
fi
card=$(i2crdtype)
if [ "${card}" != "02"  -a "${card}" != "03" -a "${card}" != "04" ]; then
    echo "Hardware without FPGA."
else
	OSType=$(cat /etc/issue | head -n 1 | cut -d " " -f 1 | awk '{print tolower($0)}')

	if [ $OSType = "centos" ]; then
		echo "CentOS detected"
	elif [ $OSType = "red" ]; then
		echo "RedHat detected"
	elif [ $OSType = "ubuntu" ]; then
		echo "Ubuntu detected"
			echo "Check software..."
			if ! dpkg -s omg-userfs >/dev/null 2>&1; then
				echo "Error: userfs missing. Perform omgupdate first."
				exit 2
			fi
			if ! dpkg -s omg-modules >/dev/null 2>&1; then
				echo "Error: modules missing. Perform omgupdate first."
				exit 2
			fi
			if ! dpkg -s omg-utilities >/dev/null 2>&1; then
				echo "Error: utilities missing. Perform omgupdate first."
				exit 2
			fi
	else
		echo "unknown OS"
			exit 3
	fi

	if ! which omgstopall > /dev/null; then
		echo "Error: omgstopall misssing. Perform omgupdate first."
		exit 2
	fi
	if ! which wdstop > /dev/null; then
		echo "Error: wdstop missing. Perform omgupdate first."
		exit 2
	fi
	if ! which cfgFlash > /dev/null; then
		echo "Error: cfgFlash  misssing. Perform omgupdate first."
		exit 2
	fi

	echo "Stopping software..."
	omgstop 2>&1 >/dev/null
	wdstop 2>/dev/null

	for i in $(getchannels -a 2>&1| grep '^0\.[a-z]/X/CFG');do
		hw=$(echo $i | cut -d '.' -f 2 | cut -d '/' -f 1)
		case "${hw}" in
		"a")
			device="/dev/tdmSpi8" 
			;;
		"b")
			device="/dev/tdmSpi108" 
			;;
		"c")
			device="/dev/tdmSpi208" 
			;;
		"d")
			device="/dev/tdmSpi308" 
			;;
		"e")
			device="/dev/tdmSpi408" 
			;;
		"f")
			device="/dev/tdmSpi508" 
			;;
		"g")
			device="/dev/tdmSpi608" 
			;;
		"h")
			device="/dev/tdmSpi708" 
			;;
		*)
			echo "cant calculate apropiate device!"
			exit 3
			;;
		esac
		
		let version=0x$(echo fpga_version | confTool /dev/ilid ${i}| head -1| cut -f 3 -d ' ')
		if [ -z "${version}" ]; then
		echo "Error: Cannot detect fpga version."
		exit 1
		fi

		if i2cinfo -l | head | grep -q OMGA; then
			echo "it's an OMGA"
			let versiontoflash=0x010b
			let minversiontoflash=0x010b
		elif [ "${card}" = "02" ]; then
			echo "it's an S2Me"
			let versiontoflash=0x0109
			let minversiontoflash=0x0106
		elif [ "${card}" = "03" ]; then 
			echo "it's an MGB"
			let versiontoflash=0x0109
			let minversiontoflash=0x106
		elif [ "${card}" = "04" ]; then 
			echo "it's an 4S2M"
			let versiontoflash=0x010a
			let minversiontoflash=0x0106
		else
			echo "Error: Can't flash due to wrong hardware."
			exit 1;
		fi

		#printf "            version is: %04X\n" ${version}
		#printf "   version to flash is: %04X\n" ${versiontoflash}
		#printf "good enough version is: %04X\n" ${minversiontoflash}

		if [ -z "${version}" -o ${version} -lt ${minversiontoflash} ];then
			flashit=1
		fi
	done

	if [ "${flashit}" == "1" ];then
		read -p "<notify>You need an FPGA update. After update it is absolutely necessary to powercycle your box. [Continue/Abort/Default=Abort]?" answer
		if [ "${answer}" != "Continue" -a "${answer}" != "C" -a "${answer}" != "c" ]; then
			echo "Aborting"
			exit 1
		fi
	fi
fi


rpmlist=$(find ${BASE} -name '*.rpm')

if test -z "${rpmlist}"; then
    echo "No Packages found."
    exit 1
fi

grep -q "exit \$RETVAL" /etc/init.d/omgWebInterface
if [ "$?" == 0 ]; then
    echo "repairing defective webstart script"
    sed -i 's/exit \$RETVAL/exit 0/' /etc/init.d/omgWebInterface
fi


yum -y --disablerepo=* remove omg omg-modules || exit 1
yum -y --disablerepo=* install ${rpmlist} || exit 1

exit 0
