// Generated by CoffeeScript 1.9.0

/*
* omg-producermanager
*
* Copyright (c) 2014 Johannes Noe
* Licensed under the MIT license.
 */
'use strict';
var aggConfig, allproducers, debug, environment, getChanges, handleCommand, handleProducerData, handleSubsription, host, logger, objectify, omg, onPullSocket, onReplySocket, os, path, producerOutputCache, publishSocket, pullSocket, replySocket, runningIntervals, setCacheAndPublishDiff, subscriptions, zmq, _;

zmq = require('zmq');

os = require('os');

path = require('path');

_ = require('lodash');

debug = require("debug")("omg-producermanager");

objectify = require("./objectify");

getChanges = require("./getChanges");

global.appConfig = require("/data/config.json");

environment = require("/data/config.json").environment;

logger = require("omg-logger");

omg = {};

allproducers = ['channels', 'cpuload', 'statics', 'diskspace', 'coredumpcheck', 'gatelogs', 'tcpdump', 'memoryusage', 'isdnconnectionsstate', 'dnsconfig', 'iproutingtable', 'scansipdestinations', 'testcall', 'testfax', 'uptime', 'systeminfo', 'licenses', 'callinfo', 'hostinspection', 'configurator', 'messagingservers', 'numberlist'];

_.each(allproducers, function(producerName) {
  return omg[producerName] = require("omg-" + producerName);
});

aggConfig = require('/data/config.json').agg;

host = aggConfig.protocol + "://" + aggConfig.address + ":";

replySocket = zmq.socket("rep");

replySocket.bindSync("" + host + aggConfig.reqRep);

pullSocket = zmq.socket("pull");

pullSocket.bindSync("" + host + aggConfig.subscriberPoolPort);

publishSocket = zmq.socket("pub");

publishSocket.bindSync("" + host + aggConfig.pubSub);

producerOutputCache = {};

subscriptions = {};

runningIntervals = {};

setCacheAndPublishDiff = function(producer, message) {
  var diff;
  message = objectify(message);
  if (!producerOutputCache[producer]) {
    producerOutputCache[producer] = {};
  }
  diff = getChanges(producerOutputCache[producer], message);
  console.log(producer, diff);
  if (!_.isEmpty(diff)) {
    publishSocket.send([producer, JSON.stringify(diff)]);
  }
  return producerOutputCache[producer] = message;
};

handleSubsription = function(requestObject) {
  var action, error, producer;
  producer = requestObject.producer;
  action = requestObject.action;
  switch (action) {
    case "subscribe":
      try {
        omg[producer][action]();
        logger.debug("subscribing to " + producer);
        return debug("subscribing to " + producer);
      } catch (_error) {
        error = _error;
        return logger.info("can not subscribe to " + producer + "; " + error.message);
      }
  }
};

handleProducerData = function(pushObject) {
  var data, error, newData, producer, reset, status;
  producer = pushObject.producer;
  data = pushObject.data;
  reset = pushObject.reset;
  status = pushObject.status;
  error = pushObject.error;
  if (!producer) {
    logger.error("producer name must be specified " + (JSON.stringify(pushObject)));
    return;
  }
  if (reset) {
    logger.debug("resetting cache of " + producer);
    producerOutputCache[producer] = {};
  }
  newData = {};
  if (data) {
    newData.outputMessage = data;
  }
  if (status) {
    newData.status = status;
  }
  if (error) {
    newData.error = error;
  }
  return publishSocket.send([producer, JSON.stringify(newData)]);
};

handleCommand = function(requestObject) {
  var action, args, producer;
  producer = requestObject.producer;
  action = requestObject.action;
  args = requestObject.args;
  if (!(producer && action)) {
    logger.warn("Producer or action was not defined");
    return;
  }
  if (args) {
    if (!omg[producer][action]) {
      logger.warn(producer + " has no method " + action);
      return;
    }
    return omg[producer][action].apply(omg[producer], args);
  } else {
    return omg[producer][action]();
  }
};

onPullSocket = function(requestObject) {
  var error, topic;
  try {
    requestObject = JSON.parse(requestObject);
  } catch (_error) {
    error = _error;
    logger.error("Can not parse message: " + requestObject);
    return;
  }
  topic = requestObject.topic;
  if (topic === "subscription") {
    handleSubsription(requestObject);
  }
  if (topic === "command") {
    handleCommand(requestObject);
  }
  if (topic === "data") {
    return handleProducerData(requestObject);
  }
};

pullSocket.on("message", onPullSocket);

onReplySocket = function(requestObject) {
  var action, args, callback, err, newArgs, producer;
  try {
    requestObject = JSON.parse(requestObject);
    producer = requestObject.producer;
    action = requestObject.action;
    args = requestObject.args || [];
    if (_.isString(args)) {
      newArgs = [];
      newArgs.push(args);
      args = newArgs;
    }
  } catch (_error) {
    err = _error;
    replySocket.send([producer || "error", JSON.stringify(err)]);
    logger.error(err);
    return;
  }
  debug(JSON.stringify(requestObject));
  callback = function(err, result) {
    var data, error, response, status;
    producer = result.producer;
    data = result.data;
    status = result.status;
    error = result.error;
    response = {};
    if (data) {
      response.outputMessage = data;
    }
    if (status) {
      response.status = status;
    }
    if (error) {
      response.error = error;
    }
    if (result) {
      replySocket.send([producer, JSON.stringify(response)]);
    }
    if (err) {
      return replySocket.send([producer, JSON.stringify(err)]);
    }
  };

  /* remove this soon */

  /*/ */
  args.push(callback);
  try {
    return omg[producer][action].apply(omg[producer][action], args);
  } catch (_error) {
    err = _error;
    replySocket.send([producer, JSON.stringify(producer + "; Error; " + err)]);
  }
};

replySocket.on("message", onReplySocket);
